/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect } from "react";
// Imports from local files
import TableData from "../../../Components/Components/Table";
import SnackBar from "../../../../Components/SnackbarComponent";
import AlertDialog from "../../../Components/Components/AlertDialog";
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import MobileList from "../../../Components/Components/MobileComponents/SimpleList"
import MobilePageHeader from "../../../Components/Components/MobileComponents/MobilePageHeader"
import PageHeader from "../../../Components/Components/PageHeader"
// Imports about icon
import FilterListRoundedIcon from "@material-ui/icons/FilterListRounded";
import AddRoundedIcon from "@material-ui/icons/AddRounded";
import UpdateRoundedIcon from "@material-ui/icons/UpdateRounded";
import TableBody from "@material-ui/core/TableBody";
import TableCell from "@material-ui/core/TableCell";
import TableRow from "@material-ui/core/TableRow";
import IconButton from "@material-ui/core/IconButton";
import VisibilityIcon from "@material-ui/icons/Visibility";
import DeleteIcon from "@material-ui/icons/Delete";
// Import from material-ui
import { withStyles } from "@material-ui/core/styles";
import Paper from "@material-ui/core/Paper";
import Button from "@material-ui/core/Button";
import Grid from "@material-ui/core/Grid";
import { CircularProgress } from "@material-ui/core";
import TextField from "@material-ui/core/TextField";
// services
import { getRequest, deleteRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
//Filters
import { Url, DeleteFilter } from "../../../Filters";
//router 
import { Link, useHistory } from 'react-router-dom';
import styled from "styled-components"
import AccountBalanceRoundedIcon from "@material-ui/icons/AccountBalanceRounded";

const StyledTableCell = withStyles((theme) => ({
  head: {
    backgroundColor: theme.palette.common.black,
    color: theme.palette.common.white,
  },
  body: {
    fontSize: 14,
  },
}))(TableCell);

const StyledTableRow = withStyles((theme) => ({
  root: {
    "&:nth-of-type(odd)": {
      backgroundColor: theme.palette.action.hover,
    },
  },
}))(TableRow);

const Institutions = () => {
  const WINDOW_WIDTH = window.innerWidth
  const ADD_ONE_LENGHT = [""];

  const router = useHistory()

  const [invertList, setInvertList] = useState(false)
  const [error, setError] = useState(null); //Necessary to consult the API, catch errors
  const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete
  const [items, setItems] = useState([]); //Necessary to consult the API, data
  const [showFilter, setShowFilter] = useState(false)
  const [currPage, setCurrPage] = useState(0)
  const [deleteItem, setDeleteItem] = useState({}); //Delete Item
  const [isLoadingToDelete, setIsLoadingToDelete] = useState(null);
  const [isLoadingMoreItems, setIsLoadingMoreItems] = useState(false);

  const [openAlertDialog, setOpenAlertDialog] = useState(false);

  const [snackInfo, setSnackInfo] = useState({
    message: "",
    icon: "",
    open: false,
    color: "",
  });

  const [search, setSeacrh] = useState("");
  const [valueOfSearch, setValueOfSearch] = useState("")
  const [city, setCity] = useState("");
  const [valueOfCity, setValueOfCity] = useState("")
  const [country, setCountry] = useState("");
  const [valueOfCountry, setValueOfCountry] = useState("")
  const [description, setDescription] = useState("");
  const [valueOfDescription, setValueOfDescription] = useState("")

  const OnChangeSearchHandler = (e) => {
    setValueOfSearch(e.target.value);
  };

  const onChangeCityHandler = (e) => {
    setValueOfCity(e.target.value);
  };

  const onChangeCountryHandler = (e) => {
    setValueOfCountry(e.target.value);
  };

  const onChangeDescriptionHandler = (e) => {
    setValueOfDescription(e.target.value);
  };
  // **************** About the PopOverMenu ****************

  //Controlls the state of the Alert Dialog
  const HandleStateAlertDialog = (i) => {
    const obj = { ...items[i] };
    setDeleteItem(obj);
    setOpenAlertDialog(!openAlertDialog);
  };

  //Controlls the state and the informations of the snack
  const HandleSnack = (message, state, icon, color) => {
    setSnackInfo({
      message: message,
      icon: icon,
      open: state,
      color: color,
    });
  };

  //Defines which row must show the circular progress
  const HandleStateCircularProgress = (i) => {
    setIsLoadingToDelete(i);
  };

  //Called when user want to delete one institution
  async function DeleteHandler() {
    const id = deleteItem.id;
    HandleStateAlertDialog(null);
    deleteRequest(
      DeleteFilter("institutions", id),
      (data) => {
        if (data.errors)
          HandleSnack("Ocorreu algum erro!", true, "warning", "#FA8072");
        else {
          HandleSnack(
            "A instituição foi deletada com sucesso",
            true,
            "success",
            "#228B22"
          );
          setCurrPage(0)
          removeItemFromList(id);
          HandleStateCircularProgress(null);
        }
      },
      (error) => {
        HandleSnack("Ocorreu algum erro!", true, "warning", "#FA8072");
        HandleStateCircularProgress(null);
      }
    )
  }

  const removeItemFromList = (itemId) => {
    let index = -1;
    for (let i = 0; i < items.length; i++) {
      const element = items[i];
      if (element.id === itemId) {
        index = i
        break
      }
    }
    if (index !== -1) {
      const cpyItems = [...items]
      cpyItems.splice(index, 1)
      setItems(cpyItems)
    }
  }

  const cleanArrayAndInvert = () => {
    setInvertList(!invertList)
    setCurrPage(0)
    setItems([])
  }

  const buildUrl = () => {
    return Url("institutions", `"name" : "${search}", "description" : "${description}", "city" : "${city}", "country" : "${country}"`, currPage, invertList ? "ASC" : "DESC")
  }

  useEffect(() => {
    if (currPage === 0)
      setIsLoaded(false)
    else
      setIsLoadingMoreItems(true)
    getRequest(
      buildUrl(),
      (data, header) => {
        const arrData = [...data]
        if (arrData.length === 0) {
          HandleSnack('Não há mais dados para serem carregados', true, 'warning', '#FFC125')
        } else {
          const arrItems = [...items]
          if (currPage === 0) {
            setItems(arrData.concat(ADD_ONE_LENGHT))
          }
          else {
            arrItems.pop(); //Deleting the last position, that was used to display the button of load more items 
            const arrResult = arrItems.concat(arrData)
            setItems(arrResult.concat(ADD_ONE_LENGHT))
          }
        }
        setIsLoaded(true)
        setIsLoadingMoreItems(false)
      },
      (error) => {
        HandleSnack('Erro ao carregar os dados', true, 'warning', '#FA8072')
        setIsLoadingMoreItems(false)
        setIsLoaded(true)
        setError(true)
      }
    )
  }, [currPage, description, country, search, city, invertList])

  useEffect(() => {
    setSeacrh("")
    setDescription("")
    setCountry("")
    setCity("")
    setValueOfDescription("")
    setValueOfSearch("")
    setValueOfCountry("")
    setValueOfCity("")
    setCurrPage(0)
  }, [showFilter])

  if (error) {
    return <div>Error: {error.message}</div>;
  }
  else if (!isLoaded) {
    return <LoadingSpinner text="Carregando..." />
  } else {
    //Words that defines that column
    const topTable = [
      "ID",
      "NOME",
      "DESCRIÇÃO",
      "CIDADE",
      "PAÍS",
      "VISUALIZAR",
      "DELETAR",
    ];

    //Field of the Filter
    const TextFieldOfTheFilter = [
      {
        label: "Pesquisar",
        value: valueOfSearch,
        helperText: "Ao digitar, retire o foco do campo",
        onChange: (event) => OnChangeSearchHandler(event),
        onBlur: (event) => setSeacrh(event.target.value)
      },
      {
        label: "Cidade",
        value: valueOfCity,
        helperText: "Ao digitar, retire o foco do campo",
        onChange: (event) => onChangeCityHandler(event),
        onBlur: (event) => setCity(event.target.value)
      },
      {
        label: "País",
        value: valueOfCountry,
        helperText: "Ao digitar, retire o foco do campo",
        onChange: (event) => onChangeCountryHandler(event),
        onBlur: (event) => setCountry(event.target.value)

      },
      {
        label: "Descrição",
        value: valueOfDescription,
        helperText: "Ao digitar, retire o foco do campo",
        onChange: (event) => onChangeDescriptionHandler(event),
        onBlur: (event) => setDescription(event.target.value)
      },
    ];

    if (WINDOW_WIDTH <= 977) {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() =>
              setSnackInfo({
                message: "",
                icon: "",
                open: false,
                color: "",
              })
            }
          />
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
          {/************** Start of the header **************/}
          <MobilePageHeader
            title="Instituições"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Filtrar",
                isLoading: false,
                func: () => {
                  setShowFilter(!showFilter);
                },
                icon: <FilterListRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/InstitutionCreate')
                },
                icon: <AddRoundedIcon />
              }
            ]}
          >
            {
              showFilter &&
              <Grid item xs={12}>
                <Grid container justify="space-between" spacing={3}>
                  {TextFieldOfTheFilter.map((field, index) => (
                    <Grid item key={field.label}>
                      <TextField
                        id={index}
                        label={field.label}
                        value={field.value}
                        helperText={field.helperText}
                        onChange={field.onChange}
                        onBlur={field.onBlur}
                      />
                    </Grid>
                  ))}
                </Grid>
              </Grid>
            }
          </MobilePageHeader>
          {/************** End of the header **************/}

          <div style={{ height: "2em" }}></div>

          {/************** Start of display data in table **************/}
          {items.map((row, index) =>
            index === items.length - 1 ? (
              <StyledDivButton
                key={new Date().toISOString() + row.created_at}
              >
                <Button
                  color="primary"
                  variant="text"
                  // disabled={isLoadingMoreItems}
                  startIcon={<AddRoundedIcon />}
                  disabled={isLoadingMoreItems}
                  onClick={() => {
                    setCurrPage(currPage + 1)
                  }}
                >
                  {isLoadingMoreItems ? (
                    <CircularProgress size={24} />
                  ) : (
                      "Carregar mais itens"
                    )}
                </Button>
              </StyledDivButton>
            ) : (
                <>
                  <MobileList
                    key={new Date().toISOString() + row.created_at}
                    title={row.name}
                    subtitle={row.id}
                    backColor={"#ff7f00"}
                    avatar={<AccountBalanceRoundedIcon />}
                    href={`/admin/institution?institution=${row.id}`}
                    reset={() => {

                    }}
                    data={
                      [
                        {
                          title: "Descrição",
                          subtitle: row.description ? row.description : "Sem dado"

                        },
                        {
                          title: "Cidade",
                          subtitle: row.city ? row.city : "Sem dado"

                        },
                        {
                          title: "País",
                          subtitle: row.country ? row.country : "Sem dado"
                        },
                        {
                          title: "Deletar",
                          subtitle:
                            <Button
                              variant="contained"
                              color="secondary"
                              onClick={() => {
                                HandleStateAlertDialog(index);
                                HandleStateCircularProgress(index);
                              }}
                              startIcon={<DeleteIcon />}
                            >
                              Deletar
                            </Button>
                        }
                      ]
                    }
                  />
                  <div style={{ height: "0.5em" }} />
                </>
              )
          )}
          {/************** End of display data in table **************/}

          {/* This alert will be displayed if the user click to delete an institution */}
        </div>
      )
    }
    else {
      return (
        <div>
          <SnackBar
            severity={snackInfo.icon}
            text={snackInfo.message}
            snackbarOpen={snackInfo.open}
            color={snackInfo.color}
            handleClose={() =>
              setSnackInfo({
                message: "",
                icon: "",
                open: false,
                color: "",
              })
            }
          />
          {/************** Start of the header **************/}
          <PageHeader
            title="Instituições"
            actions={[
              {
                name: "Atualizar",
                isLoading: false,
                func: () => {
                  setCurrPage(0)
                },
                icon: <UpdateRoundedIcon />
              },
              {
                name: "Filtrar",
                isLoading: false,
                func: () => {
                  setShowFilter(!showFilter);
                },
                icon: <FilterListRoundedIcon />
              },
              {
                name: "Novo",
                isLoading: false,
                func: () => {
                  router.push('/admin/InstitutionCreate')
                },
                icon: <AddRoundedIcon />
              }
            ]}
          >
            {
              showFilter &&
              <Grid item xs={12}>
                <Grid container justify="space-between" spacing={3}>
                  {TextFieldOfTheFilter.map((field, index) => (
                    <Grid item key={field.label}>
                      <TextField
                        id={index}
                        label={field.label}
                        value={field.value}
                        helperText={field.helperText}
                        onChange={field.onChange}
                        onBlur={field.onBlur}
                      />
                    </Grid>
                  ))}
                </Grid>
              </Grid>
            }
          </PageHeader>
          {/************** End of the header **************/}

          <div style={{ height: "2em" }}></div>

          {/************** Start of display data in table **************/}
          <TableData top={topTable} onIconPressed={cleanArrayAndInvert}>
            <TableBody>
              {items.map((row, index) =>
                index === items.length - 1 ? (
                  <StyledTableRow key={new Date().toISOString() + row.created_at}>
                    {/* Button to load more data */}
                    <StyledTableCell>
                      <Button
                        color="primary"
                        variant="text"
                        startIcon={<AddRoundedIcon />}
                        disabled={isLoadingMoreItems}
                        onClick={() => {
                          setCurrPage(currPage + 1)
                        }}
                      >
                        {isLoadingMoreItems ? (
                          <CircularProgress size={24} />
                        ) : (
                            "Carregar mais itens"
                          )}
                      </Button>
                    </StyledTableCell>
                  </StyledTableRow>
                ) : (
                    <StyledTableRow key={new Date().toISOString() + index}>
                      <StyledTableCell component="th" scope="row">
                        {row.id}
                      </StyledTableCell>
                      <StyledTableCell align="right">{row.name}</StyledTableCell>
                      <StyledTableCell align="right">
                        {row.description}
                      </StyledTableCell>
                      <StyledTableCell align="right">{row.city}</StyledTableCell>
                      <StyledTableCell align="right">
                        {row.country}
                      </StyledTableCell>
                      <StyledTableCell align="right">
                        <Link to={`/admin/institution?institution=${row.id}`}>
                          <IconButton>
                            <VisibilityIcon style={{ fill: "#00bcd4" }} />
                          </IconButton>
                        </Link>
                      </StyledTableCell>
                      <StyledTableCell align="right">
                        {isLoadingToDelete === index ? (
                          <CircularProgress size={24} color="primary" />
                        ) : (
                            <IconButton
                              onClick={() => {
                                HandleStateAlertDialog(index);
                                HandleStateCircularProgress(index);
                              }}
                            >
                              <DeleteIcon style={{ fill: "#FF0000" }} />
                            </IconButton>
                          )}
                      </StyledTableCell>
                    </StyledTableRow>
                  )
              )}
            </TableBody>
          </TableData>
          {/************** End of display data in table **************/}

          {/* This alert will be displayed if the user click to delete an institution */}
          <AlertDialog
            open={openAlertDialog}
            OnDelete={DeleteHandler}
            deleteItem={deleteItem}
            HandleClose={() => {
              setOpenAlertDialog(false);
              HandleStateCircularProgress(null);
            }}
          />
        </div>
      )
    }
  }
};

export default Institutions;

const StyledDivButton = styled(Paper)`
    width : 100%;
    display : flex; 
    justify-content : center; 
    align-items : center; 
`

