/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useEffect, useContext } from 'react';
import styled from 'styled-components';
import Grid from '@material-ui/core/Grid';
import { Link } from 'react-router-dom';
import { Store } from '../Store';

import Banner3 from '../img/ajuda/Banner3.png';
import CardPublicando from '../Components/HelpCenter/Cards/CardPublicando';
import CardEncontrando from '../Components/HelpCenter/Cards/CardEncontrando';
import CardParticipando from '../Components/HelpCenter/Cards/CardParticipando';
import CardGerenciando from '../Components/HelpCenter/Cards/CardGerenciando';

function HelpCenter(props) {
  const { state } = useContext(Store)
  let windowWidth = window.innerWidth

  useEffect(() => { window.scrollTo(0, 0) }, [])

  return (
    <div style={{ backgroundColor: "#f4f4f4" }}>
      <link href="https://fonts.googleapis.com/css?family=Pompiere|Roboto:300,400&display=swap" rel="stylesheet" />

      <Secao1 contrast={state.contrast}>
        <div className="container">
          {
            windowWidth > 420 ?
              <img src={Banner3} alt="banner3" />
              :
              <div />
          }
          <div className="conteudo">
            <div className="title">
              <h2>OLÁ! COMO PODEMOS AJUDAR?</h2>
            </div>
          </div>
        </div>
      </Secao1>

      <Secao2 contrast={state.contrast}>
        <div className="container">
          <div className="container-secao">
            <div className="conteudo">
              <div className="cabecalho">
                <h2>Tópicos de Ajuda</h2>
              </div>
              <div>
                <Grid container justify="center" style={{ margin: -8 }}>
                  <Grid item xs={12} md={5} style={{ padding: 8 }}>
                    <CardPublicando contrast={state.contrast} />
                  </Grid>
                  <Grid item xs={12} md={5} style={{ padding: 8 }}>
                    <CardEncontrando contrast={state.contrast} />
                  </Grid>
                  <Grid item xs={12} md={5} style={{ padding: 8 }}>
                    <CardParticipando contrast={state.contrast} />
                  </Grid>
                  <Grid item xs={12} md={5} style={{ padding: 8 }}>
                    <CardGerenciando contrast={state.contrast} />
                  </Grid>
                </Grid>
              </div>
            </div>
          </div>
        </div>
      </Secao2>

      <div style={{ width: "100%" }}>
        <Secao3 contrast={state.contrast}>
          <Grid style={{ height: "100%" }} container justify="center">
            <Grid style={{ backgroundColor: "#333", paddingInline: "0" }} item xs={12} md={6}>
              <iframe title="Vídeo página ajuda" src="https://player.vimeo.com/video/231609051" width="100%" height="100%" frameborder="0" allow="autoplay; fullscreen" allowfullscreen></iframe>
            </Grid>
            <Grid item xs={12} md={6}>
              <div>
                <p className="titulo-sobre-ajuda">Plataforma MEC de Recursos Educacionais Digitais</p>
                <p className="conteudo-sobre-ajuda">
                  Construa conosco a plataforma e amplie sua rede de conhecimento
                  interagindo com pessoas envolvidas com experiências que ocorrem
                  em todo o Brasil!
                </p>
              </div>
              <Grid container>
                <Grid item xs={12} md={6}>
                  <p className="links">
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '0' }
                    }}>O que é a Plataforma MEC</Link>
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '1' }
                    }}>Como foi construida a Plataforma<br />MEC?</Link>
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '3' }
                    }}>Quais são os Portais Parceiros?</Link>
                  </p>
                </Grid>
                <Grid item xs={12} md={6}>
                  <p className="links">
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '2' }
                    }}>Entendendo as 3 áreas</Link>
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '4' }
                    }}>Tipos de recursos</Link>
                    <br />
                    <Link to={{
                      pathname: 'plataforma-mec',
                      state: { value: '5' }
                    }}>Softwares específicos</Link>
                  </p>
                </Grid>
              </Grid>

            </Grid>
          </Grid>
        </Secao3>
      </div>

      <Secao4 contrast={state.contrast}>
        <div className="container">
          <div className="conteudo">
            <h2>Não encontrou o que você precisa?</h2>
            <span>Entre em contato com a nossa Central de Ajuda</span>
            <br />
            <button style={{ marginBottom: 50, marginTop: 20 }}><a href="contato">ENTRAR EM CONTATO</a></button>

          </div>
        </div>
      </Secao4>

    </div>
  );
}

export default HelpCenter;


const Secao1 = styled.div`
  background-color: ${props => props.contrast === "" ? "#00bcd4" : "black"};
  text-align: center;
  width: 100%;
  .container {
    height: 100%;
    display: flex;
    -webkit-box-pack: center;
    justify-content: center;
    -webkit-box-align: center;
    align-items: center;
    padding-left: 15px;
    margin-right: auto;
    margin-left: auto;

    img {
      position: relative;
      top: 41.6%;
      max-width: 230px;
      width: 90%;
      float: left;
      min-height: 1px;
      padding-right: 15px;
      padding-left: 15px;

    }


    .conteudo {
      display: flex;
      justify-content: center;

      .title {

      }
    }

    h2 {
      font-family: Pompiere,cursive;
      font-size: 35px;
      margin: 0;
      margin-top: 45px;
      color:#fff;
      font-weight: 500;
      line-height: 1.1;
    }

  }
`

const Secao2 = styled.div`
  background-color: ${props => props.contrast === "" ? "#f4f4f4" : "black"};
  text-align: center;
  padding-bottom: 50px
  width: 100%;
  .container {
    height: 100%;
    margin-right: auto;
    margin-left: auto;

    .container-secao {
      height: 100%;
      padding-top: 50px;
      .conteudo {
        width: 100%;
        text-align: center;
        .cabecalho {
          margin-bottom: 50px;
          text-align: center;
          h2 {
            font-size: 30px;
            font-weight: lighter;
            color: ${props => props.contrast === "" ? "#666" : "white"};
            margin: 0;
          }
        }
        .card-ajuda {

          height: 360px;
          margin-bottom: 20px


          .card {
            height: 280px;
            padding: 40px 0px;
            text-align: center;
            font-size: 14px;
            background-color:#fff;
            box-shadow: 0 1px 3px rgba(0,0,0,.12),0 1px 2px rgba(0,0,0,.24);

            h3 {
              font-size: 24px;
              font-weight: 400;
              color: #666;
              margin-top: 20px;
              margin-bottom: 10px;
              line-height: 1.1;
            }


            a {
              font-size: 15px;
              color: #666;
              text-decoration: none;
              text-align: center;

            }

            img {
              height: 62px ;
              width: 62px ;
            }

          }

          .card-rodape {
            box-sizing: border-box;

            a {
              border-radius: 0;
              width: 240.5px;
              font-size: 13px;
              font-weight: 700;
              color: #fff;
              transition: .2s ease;
              border: none;
              height: 40px;
              padding: 0 20px;
              line-height: 40px;
              background-color: #00bcd4;
              touch-action: manipulation;
              cursor: pointer;
              text-decoration: none;
              display: inline-block;
              margin-bottom: 0;
              text-align: center;
              white-space: nowrap;
              vertical-align: middle;
              touch-action: manipulation;
              cursor: pointer;
              user-select: none;
              box-shadow: 0 1px 3px rgba(0,0,0,.12),0 1px 2px rgba(0,0,0,.24);

            }
          }

        }
      }
    }
  }


`

const Secao3 = styled.div`
  padding: 0;
  text-align: center;
  background-color: ${props => props.contrast === "" ? "#fff" : "black"};
  margin-inline: auto;
  color: ${props => props.contrast === "" ? "rgba(0,0,0,0.87)" : "white"};
  width: 100%;
  .links {
    font-size: 15px;
    font-weight: lighter;
    text-align: center;
    margin: 0 0 10px;
    line-height: 1.42857143;
    a {
      color: ${props => props.contrast === "" ? "#666" : "yellow"};
      text-decoration: ${props => props.contrast === "" ? "none" : "underline"};
      font-weight: lighter;
      :hover {
        color: ${props => props.contrast === "" ? "#000" : "yellow"};
        font-weight: 500;
      }
    }
  }

  .titulo-sobre-ajuda {
    font-family: Pompiere;
    font-size: 30px;
    margin-block: 40px
  }

  .conteudo-sobre-ajuda {
    font-size: 15px;
    margin-bottom: 30px;
    margin: 0 0 10px;
  }




`

const Secao4 = styled.div`
  width: 100%;
  .container {
    height: 100%;
    background-color: ${props => props.contrast === "" ? "" : "black"};
    margin-inline: auto;
    display: flex;
    flex-direction: column;
    justify-content: center;
    .conteudo {
      width: 100%;
      text-align: center;

      h2 {
        font-size: 24px;
        font-weight: lighter;
        color: ${props => props.contrast === "" ? "#666" : "white"};
        margin-top: 20px;
        margin-bottom: 10px;
        text-align: center;
      }

      span {
        font-size: 15px;
        color: ${props => props.contrast === "" ? "#777" : "white"};
        text-align: center;
      }

      button {
        margin-top: 25px;
        color: ${props => props.contrast === "" ? "rgba(255,255,255,0.87)" : "yellow"};
        background-color: ${props => props.contrast === "" ? "rgb(255,127,0)" : "black"};
        box-shadow: 0 2px 5px 0 rgba(0,0,0,.26);
        outline: none;display: inline-block;
        position: relative;
        cursor: pointer;
        min-height: 36px;
        min-width: 88px;
        line-height: 36px;
        vertical-align: middle;
        -webkit-box-align: center;
        align-items: center;
        text-align: center;
        border-radius: 3px;
        box-sizing: border-box;
        user-select: none;
        border: ${props => props.contrast === "" ? "none" : "1px solid white"};
        padding: 0 6px;
        margin: 6px 8px;
        white-space: nowrap;
        text-transform: uppercase;
        font-weight: 500;
        font-size: 14px;
        font-style: inherit;
        font-variant: inherit;
        font-family: inherit;
        text-decoration: none;
        overflow: hidden;
        transition: box-shadow .4s cubic-bezier(.25,.8,.25,1),background-color .4s cubic-bezier(.25,.8,.25,1);
        letter-spacing: .01em;
        :hover{
          background-color: ${props => props.contrast === "" ? "" : "rgba(255,255,0,0.24)"};
        }
        a {
          color: ${props => props.contrast === "" ? "rgba(255,255,255,0.87)" : "yellow"};
          text-decoration: ${props => props.contrast === "" ? "none" : "underline"};
        }
      }
    }
  }


`
